# -*- coding: utf-8 -*-
"""
Script to convert FEM file to KEY file input to LS-DYNA. Works with both Genie- and Patran-Pre-generated inputfiles of FEM type. 

The script is provided for internal use at NTNU only without re-distribution outside NTNU.

Moss Maritime AS takes no responsibility for the accuracy of the conversion. The analyst should independently check that all nodes, elements and properties are trasferred as intended. 



Features transferred : 
Nodes - All types
Elements - Element types 15, 23, 24, 26, 28. Second order elements (8-6 nodes) reduce to linear elements when writing LS-DYNA input
Thicknesses - OK
Materials- OK
Sets - OK

NOTE : Error in beam element properties, likely related to beam orientation vector that shifts property membership

Created on Mon Dec 14 16:58:21 2015

@author: Martin Storheim, Moss Maritime

Modified for Python 3 on Jan 18 by Stian A. Hagen

"""


import numpy as np  # Import all (top-level) functions from numpy
import math
import os, sys # Import system modules
def chunkstring(string, length):
    return (string[0+i:length+i] for i in range(0, len(string), length))
   


#inputfilename = 'input/20151209_142406_T1.FEM'  # For debug/direct use

f = open('fem2keyINPUT.txt','r')
dummy = f.readline()
inputfilename = f.readline()
inputfilename = inputfilename.strip()
dummy = f.readline()
outputfilename = f.readline()
outputfilename = outputfilename.strip()
f.close()

# Create log file
logfilename = 'fem2key.log'
logfile = open(logfilename, 'w')
logfile.write('*************************\n')
logfile.write('***   FEM2KEY logfile ***\n')
logfile.write('*************************\nn')
logfile.write('Inputfile %s\n' % inputfilename)
logfile.write('Outputfile %s\n' % outputfilename)


# Print status messages for each step if value below =1
printsuppress = 0
 

#%% Read FEM file
IDENT = []          # Heading
TDMATER = []        #
MISOSEL = []        # Material properties
GELTH = []          # Element thicknesses
GNODE = []          # Nodes
GCOORD = []         # Nodal coordinates
GELMNT1Q = []       # Quad elem linear
GELMNT1T = []       # Tria elem linear
GELMNT1B = []       # Linear beam element
GELMNT1Q2 = []       # Quad elem curved
GELMNT1T2 = []       # Tria elem curved
GELREF1 = []        # Element reference to thick and mat
TDSETNAM = []       # Name of sets
TDSECT = []       # Name of beam cross sections
GSETMEMB = []     # Set members
GBEAMG = [] # General beam element data
GPIPE = [] # Pipe section properties
GBARM = [] # Flatbar section properties
GUNIVEC = [] # Local coordinate system of beam element

nmater = 0
nmat = 0
nsect = 0
nsectprop = 0
nthick = 0
nnode = 0
ncoord = 0
nelmQ = 0
nelmT = 0
nelmQ2 = 0
nelmT2 = 0
nelmB = 0
nelref = 0
nsets = 0
nsets2 = 0
npipe = 0
nbars = 0
ngvec = 0

# Parameters for unknown keywords
unknownKEY = []

dummycount = -1 # counter for lines read
logfile.write("Starting to read the file : %s\n" % inputfilename)
f=open(inputfilename,'r')
while dummycount > -2:
    dummycount = dummycount + 1
    dataRAW = f.readline()
    data = dataRAW.strip()
    columns=data.split()
    
    if 'IDENT' in dataRAW[0:8]:
        IDENT = columns[1:]
    elif 'DATE' in dataRAW[0:8]:
        for i in range(0,4):
            dummy = f.readline()
    elif 'TDMATER' in dataRAW[0:8]:
        line2 = f.readline()
        line2 = line2.strip()
        columns2 = line2.split()
        TDMATER.append([]) 
        TDMATER[nmater] = '%s %s %s %s %s' % (columns2[0], columns[1], columns[2], columns[3], columns[4])
        nmater = nmater + 1
    elif 'MISOSEL' in dataRAW[0:8]:
        line2 = f.readline()
        line2 = line2.strip()
        columns2 = line2.split()
        MISOSEL.append([]) 
        MISOSEL[nmat] = '%s %s %s %s %s %s' % (columns[1], columns[2], columns[3], columns[4], columns2[0], columns2[1])
        nmat = nmat + 1    
    elif 'TDSECT' in dataRAW[0:8]:
        line2 = f.readline()
        line2 = line2.strip()
        columns2 = line2.split()
        TDSECT.append([]) 
        TDSECT[nsect] = '%s %s %s %s %s' % (line2 , columns[1], columns[2], columns[3], columns[4])
        nsect = nsect + 1        
    elif 'GELTH' in dataRAW[0:8]:
        GELTH.append([]) 
        GELTH[nthick] = columns[1:]
        nthick = nthick + 1
    elif 'GNODE' in dataRAW[0:8]:
        GNODE.append([])
        GNODE[nnode] = columns[1:]
        nnode = nnode + 1
    elif 'GCOORD' in dataRAW[0:9]:
        GCOORD.append([])
        GCOORD[ncoord] = [dataRAW[9:24]] + list(chunkstring(dataRAW[24:-1],16))
        ncoord = ncoord + 1           
    elif 'BNBCD' in dataRAW[0:8]:
        f.readline() # Skip
    elif 'GELMNT1' in dataRAW[0:8]:
        if int(float(columns[3])) == 24:      # Quads linear
            line2 = f.readline()
            line2 = line2.strip()
            columns2 = line2.split()        
            GELMNT1Q.append([]) 
            GELMNT1Q[nelmQ] = '%s %s %s %s %s %s %s %s' % (columns[1], columns[2], columns[3], columns[4], columns2[0], columns2[1], columns2[2], columns2[3])
            nelmQ = nelmQ + 1   
        elif int(float(columns[3])) == 25:      # Quads linear
            line2 = f.readline()
            line2 = line2.strip()
            columns2 = line2.split()
            GELMNT1T.append([]) 
            GELMNT1T[nelmT] = '%s %s %s %s %s %s %s' % (columns[1], columns[2], columns[3], columns[4], columns2[0], columns2[1], columns2[2])
            nelmT = nelmT + 1   
        elif int(float(columns[3])) == 26:      # Tria curved. Nodes : corner, corner, corner, mid, mid, mid
            line2 = f.readline()
            line2 = line2.strip()
            columns2 = line2.split()
            line3 = f.readline()
            line3 = line3.strip()
            columns3 = line3.split()            
            GELMNT1T2.append([]) 
            GELMNT1T2[nelmT2] = '%s %s %s %s %s %s %s %s %s %s' % (columns[1], columns[2], columns[3], columns[4], \
                                                        columns2[0], columns2[1], columns2[2], columns2[3], \
                                                        columns3[0], columns3[1])
            nelmT2 = nelmT2 + 1    
        elif int(float(columns[3])) == 28:      # Quads curved. Nodes : corner, mid, corner, mid ....
            line2 = f.readline()
            line2 = line2.strip()
            columns2 = line2.split()
            line3 = f.readline()
            line3 = line3.strip()
            columns3 = line3.split()            
            GELMNT1Q2.append([]) 
            GELMNT1Q2[nelmQ2] = '%s %s %s %s %s %s %s %s %s %s %s %s' % (columns[1], columns[2], columns[3], columns[4], \
                                                        columns2[0], columns2[1], columns2[2], columns2[3], \
                                                        columns3[0], columns3[1], columns3[2], columns3[3])
            nelmQ2 = nelmQ2 + 1            
        elif int(float(columns[3])) == 15:      # Beam 2/node
            line2 = f.readline()
            line2 = line2.strip()
            columns2 = line2.split()
            GELMNT1B.append([]) 
            GELMNT1B[nelmB] = '%s %s %s %s %s %s' % (columns[1], columns[2], columns[3], columns[4], columns2[0], columns2[1])
            nelmB = nelmB + 1   
    elif 'GELREF1' in dataRAW[0:9]:
        line2 = f.readline()
        line2 = line2.strip()
        columns2 = line2.split()
        line3 = f.readline()
        line3 = line3.strip()
        columns3 = line3.split()        
        GELREF1.append([]) 
        GELREF1[nelref] = '%s %s %s %s %s %s %s %s %s %s %s %s' % (columns[1], columns[2], columns[3], columns[4],\
                    columns2[0], columns2[1], columns2[2], columns2[3], columns3[0], columns3[1], columns3[2], columns3[3])
        nelref = nelref+ 1    
    elif 'GBEAMG' in dataRAW[0:9]:
        line2 = f.readline()
        line2 = line2.strip()
        columns2 = line2.split()
        line3 = f.readline()
        line3 = line3.strip()
        columns3 = line3.split()        
        line4 = f.readline()
        line4 = line4.strip()
        columns4 = line4.split()                
        GBEAMG.append([]) 
        GBEAMG[nsectprop] = '%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s' % (columns[1], columns[2], columns[3], columns[4],\
                    columns2[0], columns2[1], columns2[2], columns2[3], columns3[0], columns3[1], columns3[2], columns3[3],\
                    columns4[0], columns4[1], columns4[2], columns4[3])
        nsectprop = nsectprop + 1            
    elif 'TDSETNAM' in dataRAW[0:9]:
        line2 = f.readline()
        line2 = line2.strip()
        TDSETNAM.append([]) 
        TDSETNAM[nsets] = '%s %s %s %s %s' % (columns[1], columns[2], columns[3], columns[4], line2)
        nsets = nsets + 1        
    elif 'GPIPE' in dataRAW[0:8]:
        line2 = f.readline()
        line2 = line2.strip()
        columns2 = line2.split()
        GPIPE.append([]) 
        GPIPE[npipe] = '%s %s %s %s %s %s' % ( columns[1], columns[2], columns[3], columns[4], columns2[0],  columns2[1])
        npipe = npipe + 1          
    elif 'GBARM' in dataRAW[0:8]:
        line2 = f.readline()
        line2 = line2.strip()
        columns2 = line2.split()
        GBARM.append([]) 
        GBARM[nbars] = '%s %s %s %s %s %s' % ( columns[1], columns[2], columns[3], columns[4], columns2[0],  columns2[1])
        nbars = nbars + 1          
    elif 'GUNIVEC' in dataRAW[0:9]:
        GUNIVEC.append([]) 
        GUNIVEC[ngvec] = '%s %s %s %s ' % (columns[1], columns[2], columns[3], columns[4])
        ngvec = ngvec + 1              
    elif 'GSETMEMB' in dataRAW[0:9]:
        GSETMEMB.append([])
        GSETMEMB[nsets2] = '%s %s %s %s' % (columns[1], columns[2], columns[3], columns[4])
        #print nsets2, int(math.ceil(float(GSETMEMB[nsets2].split()[0])/4)) -1, GSETMEMB[nsets2].split()[0]
        for i in range(0, int(math.ceil(float(GSETMEMB[nsets2].split()[0])/4))-1):
            # Read line
            line2 = f.readline()
            line2 = line2.strip()
            GSETMEMB[nsets2] += ' %s' % (line2)            
        nsets2 = nsets2 +1
    elif 'IEND' in dataRAW[0:8]:
        IEND = data
        logfile.write("Read to end of FEM file\n\n")
    elif dataRAW == '': # Reached end of file
        dummycount = - dummycount
    else:
        if ('    ' not in dataRAW[0:8]) and (printsuppress < 100): # Then we have an unsupported keyword
            if len(unknownKEY) == 0: # First unsuported keyword
                logfile.write('*** Unsupported keyword : %s\n' % (data.split()[0]))
                unknownKEY.append(data.split()[0])                
                printsuppress = printsuppress + 1
            else:
                if data.split()[0] not in unknownKEY:
                    logfile.write('*** Unsupported keyword : %s\n' % (data.split()[0]))
                    unknownKEY.append(data.split()[0])                
                    printsuppress = printsuppress + 1                  
                
            
f.close()

# Print summary
logfile.write('Summary : \n' )
logfile.write('Read %i nodes\n' % (nnode))
logfile.write('Read %i linear tria elements\n' % nelmT )
logfile.write('Read %i linear quad elements\n' % nelmQ )
logfile.write('Read %i qubic tria elements\n' % nelmT2 )
logfile.write('Read %i qubic quad elements\n' % nelmQ2 )
logfile.write('Read %i materials\n' % nmat)
logfile.write('Read %i thicknesses\n' % nthick)
logfile.write('Read %i sets on %i set cards\n' % (nsets, nsets2))




#%% Sort data and print to file


GELREF1ID = ['ELNO', 'MATNO', 'ADDNO', 'INTNO', 'MINTNO', 'STRANO', 'STRENO', 'STREPONO', 'GEONO/OPT', 'FIXNO/OPT', 'ECCNO/OPT', 'TRANSNO/OPT']
GNODEID = ['NODEX', 'NODENO', 'NDOF', 'ODOF']
GELMNT1ID = ['ELNOX', 'ELNO', 'ELTYP', 'ELTYAD', 'NODIN1', 'NODIN2']
MISOSELID = ['MATNO', 'YOUNG', 'POISS', 'RHO', 'DAMP', 'ALPHA']
TDMATERID = ['NAME', 'NFIELD', 'GEONO', 'CODNAM', 'CODTXT']
GCOORDID = ['NODENO', 'XCOORD', 'YCOORD', 'ZCOORD']
GELTHID = ['GEONO', 'TH', 'NINT']
GSETMEMBID = ['NFIELD', 'ISREF', 'INDEX', 'ISTYPE', 'ISORIG', 'IRMEMB1', 'IRMEMBn']
TDSECTID = ['NFIELD', 'GEONO' ,'CODNAM', 'CODTXT', 'GS-Name']
GBEAMGID =[ 'GBEAMG',  'GEONO', 'VOID', 'AREA', 'IX', 'IY', 'IZ', 'IYZ', 'WXMIN', 'WYMIN', 'WZMIN', 'SHARY', 'SHARZ', 'SHCENY', 'SHCENZ', 'SY', 'SZ']

GPIPEID =['GEONO', 'DI', 'Dy', 't', 'SFY', 'SFZ', 'NCIR', 'NRAD']
GBARMID = ['GEONO', 'HZ', 'BT', 'BB', 'SFY', 'SFZ', 'NLOBY', 'NLOBZ']
GUNIVECID = ['TRANSNO', 'UNIX', 'UNIY', 'UNIZ']

# Create property set matrix
propID = []
beampropID = []
nprop = 0
nbeamprop=0



for i in range(0,len(GELREF1)):
    matID = int(float(GELREF1[i].split()[1]))
    geoID = int(float(GELREF1[i].split()[8]))
    beamvect = int(float(GELREF1[i].split()[11]))
    if beamvect == 0:  # We have a shell element
        if len(propID) == 0:
            propID.append('%s %s %s' % (nprop + 1, matID, geoID))
            nprop = nprop + 1
        else: # Check if property already exists
            dummyexist = 0 # prop existing
            for j in range(0,nprop):
                if (int(float(propID[j].split()[1])) == matID) and (int(float(propID[j].split()[2])) == geoID):
                    dummyexist=1
            if dummyexist == 0:       
                propID.append('%s %s %s' % (nprop + 1, matID, geoID))
                nprop = nprop + 1
    elif beamvect !=0: # Then we have a beam element
#        print 'BEAM element nr %s with GEOID %s and vect %s' %(GELREF1[i].split()[0], geoID, beamvect)
        if len(beampropID) == 0:
            beampropID.append('%s %s %s %s' % (nbeamprop + 1, matID, geoID, beamvect))
            nbeamprop = nbeamprop + 1
        else: # Check if property already exists
            dummyexist = 0 # prop existing
            for j in range(0,nbeamprop):
                if (int(float(beampropID[j].split()[1])) == matID) and (int(float(beampropID[j].split()[2])) == geoID):
                    dummyexist=1
            if dummyexist == 0:       
                beampropID.append('%s %s %s %s' % (nprop + 1, matID, geoID, beamvect))
                nbeamprop = nbeamprop + 1

                

#for ref in GELREF1: print ref.split()[-4]


#%%

for i in range(0,len(GELREF1)):
    matID = int(float(GELREF1[i].split()[1]))
    geoID = int(float(GELREF1[i].split()[8]))
    beamvect = int(float(GELREF1[i].split()[11]))
# Add set reference to GELREF1 for shell elements    
    dummy=0
    for j in range(0,nprop):
        if (int(float(propID[j].split()[1])) == matID) and (int(float(propID[j].split()[2])) == geoID):
            GELREF1[i] += ' %i' % (j+1)
            dummy=1
            #print 'elem %i with mat %i and geo %i has prop %i' % (i, matID, geoID, j+1)
    if dummy ==0:
        GELREF1[i] += ' %i' % (0)
# Add set reference to GELREF1 for beam elements
    dummy=0
    for j in range(0,nbeamprop):
        if (int(float(beampropID[j].split()[1])) == matID) and (int(float(beampropID[j].split()[2])) == geoID) and (int(float(beampropID[j].split()[3])) == beamvect):
            GELREF1[i] += ' %i' % (j+1)
            dummy=1
#            print 'elem %i with mat %i and geo %i and vect %i has prop %i' % (i, matID, geoID, beamvect, j+1)
    if dummy ==0:
        GELREF1[i] += ' %i' % (0)    


#%% Fix reference if thicknesses are not numerically numbered

thickID = []

for i in range(0,len(GELTH)):
    thickID.append(int(float(GELTH[i][0])))

thicknesses = np.zeros(np.max(thickID)+1)
for i in range(0,len(GELTH)):
    thicknesses[int(float(GELTH[i][0]))] =  float(GELTH[i][1])
    
    

#%% 
setIDs = []
nsetIDs = 0

# Check if a set has multiple GSETMEMB cards (max 1024 in each)
for i in range(0,nsets2):
    setref = int(float(GSETMEMB[i].split()[1]))
    setrefind = int(float(GSETMEMB[i].split()[2]))
    #print i, setref, setrefind
    if (setrefind == 1) and (setref == 1):
        setIDs.append([setref, setrefind])
        nsetIDs = nsetIDs + 1
    elif (setrefind == 1) and (setref > 1):
        setIDs.append([setref, setrefind])  
        nsetIDs = nsetIDs + 1
    elif setrefind > 1:
        for j in range(0,nsetIDs):
            if (setIDs[j][0] == setref) and (setrefind > 1):
                setIDs[j].append(setrefind)
            
            
        


#%% Write LS-DYNA keyword file 

logfile.write('Starting to write output\n')

try:
    os.remove(outputfilename)
except OSError:
    pass


fileout=open(outputfilename,'w')
fileout.write('*KEYWORD \n')
fileout.write('*TITLE\n')
fileout.write('Imported model from SESAM FEM-file\n')
# Materials
for i in range(0,nmat):
    fileout.write('*MAT_ELASTIC\n')
#    fileout.write('%s \n' % (TDMATER[i].split()[0]) )
    fileout.write('$#     mid        ro         e        pr        da        db  not used \n')
    fileout.write('%i, %f, %g, %f, 0, 0, 0\n' %(i+1, float(MISOSEL[i].split()[3]), \
                    float(MISOSEL[i].split()[1]), float(MISOSEL[i].split()[2])  ))
#             1    7850.02.10000E110.30000001       0.0       0.0         0
# Nodes
fileout.write('*NODE \n')
fileout.write('$#   nid               x               y               z      tc      rc  \n')
for i in range(0,nnode):
    fileout.write('  %i, %f, %f, %f, 0, 0 \n' % (int(float(GNODE[i][1])), \
                   float(GCOORD[i][1]), float(GCOORD[i][2]), float(GCOORD[i][3]) ))

# Elements
fileout.write('*ELEMENT_SHELL \n')
fileout.write('$#   eid     pid      n1      n2      n3      n4      n5      n6      n7      n8 \n')
for i in range(0,nelmQ):    # Quad elements
    temp = GELMNT1Q[i].split()
    temp = list(map(float, temp))
    temp = list(map(int, temp))
    fileout.write('  %i, %i, %i, %i, %i, %i, 0, 0, 0, 0 \n' % (temp[1], int(GELREF1[temp[1]-1].split()[12]), \
                    temp[4], temp[5], temp[6], temp[7] ))
for i in range(0,nelmQ2):    # Quad elements cubic, downsampled to linear
    temp = GELMNT1Q2[i].split()
    temp = map(float, temp)
    temp = map(int, temp)
    fileout.write('  %i, %i, %i, %i, %i, %i, 0, 0, 0, 0 \n' % (temp[1], int(GELREF1[temp[1]-1].split()[12]), \
                    temp[4], temp[6], temp[8], temp[10] ))
                    
for i in range(0,nelmT):    # Tria elements
    temp = GELMNT1T[i].split()
    temp = list(map(float, temp))
    temp = list(map(int, temp))
    fileout.write('  %i, %i, %i, %i, %i, 0, 0, 0, 0, 0 \n' % (temp[1], int(GELREF1[temp[1]-1].split()[12]), \
                    temp[4], temp[5], temp[6] ))
for i in range(0,nelmT2):    # Tria elements
    temp = GELMNT1T2[i].split()
    temp = map(float, temp)
    temp = map(int, temp)
    fileout.write('  %i, %i, %i, %i, %i, 0, 0, 0, 0, 0 \n' % (temp[1], int(GELREF1[temp[1]-1].split()[12]), \
                    temp[4], temp[5], temp[6] ))
                    
fileout.write('*ELEMENT_BEAM_ORIENTATION \n')                    
fileout.write('$#   eid     pid      n1      n2      n3     rt1     rr1     rt2     rr2   local \n')                    
for i in range(0,nelmB):    # Tria elements
    temp = GELMNT1B[i].split()
    temp = map(float, temp)
    temp = map(int, temp)
    tsect = int(float(GELREF1[temp[1]-1].split()[11])) # SEction ID for orientation vector
    if int(float(GUNIVEC[tsect-1].split()[0])) == tsect:
        beamvect = map(float,GUNIVEC[tsect-1].split())
    else: 
        print('Error in beam section')
        beamvect=[0, 0, 0, 0]
    
    fileout.write('  %i, %i, %i, %i, 0, 0, 0, 0, 0, 0 \n' % (temp[1], nprop +1 + int(GELREF1[temp[1]-1].split()[13]), \
                    temp[4], temp[5] ))                    
    fileout.write('  %.5f, %.5f, %.5f  \n'  % (beamvect[1], beamvect[2],beamvect[3] ))  # Beam orientation
                    
# Properties
propcount = 0
for i in range(0,nprop):  # Shells
    propcount = propcount + 1
    fileout.write('*PART\n')
    fileout.write('$#                                                                         title\n')
    fileout.write('Part %i\n' % (propcount))
    fileout.write('$#     pid     secid       mid     eosid      hgid      grav    adpopt      tmid\n')
    fileout.write('  %i, %i, %i, 0, 0, 0, 0, 0\n' % ( propcount, propcount, int(float(propID[i].split()[1])) ))

    fileout.write('*SECTION_SHELL\n')
    fileout.write('$#   secid    elform      shrf       nip     propt   qr/irid     icomp     setyp\n')
    fileout.write('  %i, %i, %f, %i, 1, 0, 0, 1\n' % ( propcount, 2, 0.833, 5 ))
    fileout.write('$#      t1        t2        t3        t4      nloc     marea      idof    edgset\n')
    #thick = float(GELTH[int(float(propID[i].split()[2]))-1][1])
    thick = thicknesses[int(float(propID[i].split()[2]))]
    fileout.write('  %f, %f, %f, %f, 0, 0, 0, 0\n' % (thick, thick, thick, thick))    
    
    
for i in range(0,nbeamprop):  # Beams
    propcount = propcount + 1
    print('Writing beam %i') % propcount
    fileout.write('*PART\n')
    fileout.write('$#                                                                         title\n')
    fileout.write('Part %i\n' % (propcount))
    fileout.write('$#     pid     secid       mid     eosid      hgid      grav    adpopt      tmid\n')
    fileout.write('  %i, %i, %i, 0, 0, 0, 0, 0\n' % ( propcount, propcount, int(float(beampropID[i].split()[1])) ))

    temptest=0
    # Check section type
    for pipe in GPIPE: 
        if int(float(beampropID[i].split()[2])) == int(float(pipe.split()[0])): # Then we have a pipe section
            temptest=1
            print('pipe section')
            fileout.write('*SECTION_BEAM\n')
            fileout.write('$#   secid    elform      shrf       nip     propt   qr/irid     icomp     setyp\n')
            fileout.write('  %i, %i, %f, 2, 1, 0, 0. \n' % ( propcount, 1, 1.0  ))
            fileout.write('$#     ts1       ts2       tt1       tt2     nsloc     ntloc     \n')
            fileout.write('  %f, %f, %f, %f, 0, 0 \n' % (float(pipe.split()[2]),float(pipe.split()[2]) , float(pipe.split()[1]), float(pipe.split()[1])))   
    for pipe in GBARM: 
        if int(float(beampropID[i].split()[2])) == int(float(pipe.split()[0])): # Then we have a pipe section
            temptest=1
            print('bar section')
            fileout.write('*SECTION_BEAM\n')
            fileout.write('$#   secid    elform      shrf       nip     propt   qr/irid     icomp     setyp\n')
            fileout.write('  %i, %i, %f, 2, 0, 0, 0. \n' % ( propcount, 1, 1.0  ))
            fileout.write('$#     ts1       ts2       tt1       tt2     nsloc     ntloc     \n')
            fileout.write('  %f, %f, %f, %f, 0, 0,\n' % (float(pipe.split()[2]),float(pipe.split()[2]) , float(pipe.split()[1]), float(pipe.split()[1])))               
    if temptest==0: print('Error, beam not found')



    

# Sets
setteller = 0
for i in range(0,nsetIDs):
    setname = TDSETNAM[i].split()[4]
    setnumber = int(float(TDSETNAM[i].split()[1]))
    setindex = setIDs[i][-1]
    fileout.write('*SET_SHELL_LIST_TITLE\n')
    fileout.write('%s \n' % (setname) )
    fileout.write('$#     sid       da1       da2       da3       da4   \n')
    fileout.write('%i, 0., 0., 0., 0.\n' % (setnumber) )
    fileout.write('$#    eid1      eid2      eid3      eid4      eid5      eid6      eid7      eid8 \n')
    elems = []
    if int(float(GSETMEMB[setteller].split()[3])) == 2:  # Then this is an element set
        for j in range(0,setindex):
            dummy = GSETMEMB[setteller].split()[5:]
            for k in range(0,len(dummy)):
                elems.append(int(float(dummy[k])))
            setteller = setteller +1    
        #print setname, setnumber, setindex, setteller
        
        linecounter = 0
        for k in range(0,len(elems)):
            if k==0:
                fileout.write('  %i' % (elems[k]))
                linecounter = linecounter + 1
            else:
                if linecounter == 7:
                    linecounter = 0
                    fileout.write(', %i\n' % (elems[k]))
                elif linecounter == 0:
                    fileout.write('  %i' % (elems[k]))
                    linecounter = linecounter + 1                       
                else:
                    fileout.write(', %i' % (elems[k]))
                    linecounter = linecounter + 1
                    
        fileout.write(' \n')    
fileout.write('*END')

fileout.close()



logfile.write('Finished writing output\n\n' )
logfile.write('FEM2KEY successfully completed\n' )

#%% 
logfile.close()

