%--------------------------------------------------------------------------------
%---------------Subroutine for 3D external mechanics  ---------------------------
%---------------  by Dr. Z.Liu, March,2013  -------------------------------------
%--------------------------------------------------------------------------------
function [tt,ttm,dvv,ve_af,ve_bf,flag,miu,mass1,mass2]=stronge3d(Mass1,Mass2,Am,Bm,Amr,Bmr,Ra,Rb,alpha,gama,betap,cp_a,cp_b,res,miu0,ve_a,ve_b)
%
% INPUT PARAMETERS
% Mass1 : mass of object a, no added mass included [kg]
% Mass2 : mass of object b, no added mass included [kg]
% Am    : translational added mass of object a under body frame of object a [-]
% Bm    : translational added mass of object b under body frame of object b
% Amr   : rotational added mass of object a under body frame of object a
% Bmr   : rotational added mass of object b under body frame of object b
% Ra    : inertia radius square of object a under body frame of object a
%         [m2]
% Rb    : inertia radius square of object a under body frame of object b
%         [m2]
% alpha : waterline angle [deg]
% gama  : angle between body frame of object a and b [deg]
% betap : normal frame angle [deg]
% cp_a  : collision point under body frame of object a, array (3x1)
% cp_b  : collision point under body frame of object b,array (3x1)
% ve_a  : velocity of object a under body frame of object a,array (3x1)
% ve_b  : velocity of object b under body frame of object b,array (3x1)
% res   : restitution factor, res=0 (fully plastic), res=1 (fully elastic)
% miu0  : the static friction
%----------------------------------------------------------------------------------
% OUTPUT PARAMETERS
% tt    : total dissipated energy [J]
% ttm   : an array for dissipated energy in each direction [J]
% dvv   : relative velocity increase under the local frame n1n2n3
% ve_af : velocity after impact of object a under body frame of object a
% ve_bf : velocity after impact of object b under body frame of object b
% flag  : stick (1) or slide (2)
% miu   : static friction factor between object a and b
% mass1 : mass matrix for object a
% mass2 : mass matrix for object b
%
%----------------------------------------------------------------------------------
mass1=[1+Am(1) 0 0; 0 1+Am(2) 0; 0 0 1+Am(3)]*Mass1; % mass matrix for object a
mass2=[1+Bm(1) 0 0; 0 1+Bm(2) 0; 0 0 1+Bm(3)]*Mass2; % mass matrix for object b
rxa=Ra(1); % gyration radius square for a
rya=Ra(2); % gyration radius square for a
rza=Ra(3); % gyration radius square for a
rxb=Rb(1); % gyration radius square for b
ryb=Rb(2); % gyration radius square for b
rzb=Rb(3); % gyration radius square for b
Itrx1=[(1+Amr(1))*rxa 0 0;0 (1+Amr(2))*rya 0; 0 0 (1+Amr(3))*rza]*Mass1; % Inertia matrix for a
Itrx2=[(1+Bmr(1))*rxb 0 0;0 (1+Bmr(2))*ryb 0; 0 0 (1+Bmr(3))*rzb]*Mass2; % Inertia matrix for b
oba_g=[0 0 0]'; % gravity center of a under body frame of object a
obb_g=[0 0 0]'; % gravity center of b under body frame of object b
alpha=alpha/180*pi; %wanterline angle
gama =gama/180*pi;  % angle between body frame a and b
betap=betap/180*pi;
% calculate the relative impact vector under body frame a and b
rad1a=cp_a-oba_g;
rad2b=cp_b-obb_g;
% trasnformation matrix between body frame a and b
Mab=[cos(gama) sin(gama) 0;
    -sin(gama) cos(gama) 0;
    0 0 1];
% transformation matrix between local and global system for a
l=sin(alpha)*cos(betap);
m=cos(alpha)*cos(betap);
n=-sin(betap);
Mlg=[cos(alpha) -sin(alpha) 0;
    -sin(alpha)*sin(betap) -cos(alpha)*sin(betap) -cos(betap);   
    l m n];
% transformation matrix between local and global system for b
Mtr2=Mlg*Mab;
% calculate the transformed inertia matrix
Rtrx1=inv(Mlg*Itrx1*inv(Mlg));
Rtrx2=inv(Mtr2*Itrx2*inv(Mtr2));
mass1f=inv(Mlg*mass1*inv(Mlg));
mass2f=inv(Mtr2*mass2*inv(Mtr2));
% calculate the impact vector under local frame
rad1=Mlg*rad1a; 
rad2=Mtr2*rad2b;
% calculate the relative velocity under local system
rvl=Mlg*ve_a-Mtr2*ve_b;
% Input the reversed mass matrix
m11=(mass1f(1,1)+rad1(2)^2*Rtrx1(3,3)-2*rad1(2)*rad1(3)*Rtrx1(2,3)+rad1(3)^2*Rtrx1(2,2))+ ...
    (mass2f(1,1)+rad2(2)^2*Rtrx2(3,3)-2*rad2(2)*rad2(3)*Rtrx2(2,3)+rad2(3)^2*Rtrx2(2,2));
m12=(mass1f(1,2)+mass2f(1,2))+(rad1(1)*rad1(3)*Rtrx1(2,3)-rad1(3)^2*Rtrx1(2,1)-...
    rad1(1)*rad1(2)*Rtrx1(3,3)+rad1(2)*rad1(3)*Rtrx1(3,1))+...
    (rad2(1)*rad2(3)*Rtrx2(2,3)-rad2(3)^2*Rtrx2(2,1)-rad2(1)*rad2(2)*Rtrx2(3,3)+rad2(2)*rad2(3)*Rtrx2(3,1));
m13=(mass1f(1,3)+mass2f(1,3))+(rad1(1)*rad1(2)*Rtrx1(3,2)-...
    rad1(2)^2*Rtrx1(3,1)-rad1(1)*rad1(3)*Rtrx1(2,2)+rad1(2)*rad1(3)*Rtrx1(2,1))+...
    (rad2(1)*rad2(2)*Rtrx2(3,2)-rad2(2)^2*Rtrx2(3,1)-rad2(1)*rad2(3)*Rtrx2(2,2)+rad2(2)*rad2(3)*Rtrx2(2,1));
m22=(mass1f(2,2)+rad1(1)^2*Rtrx1(3,3)-2*rad1(1)*rad1(3)*Rtrx1(1,3)+rad1(3)^2*Rtrx1(1,1))+...
    (mass2f(2,2)+rad2(1)^2*Rtrx2(3,3)-2*rad2(1)*rad2(3)*Rtrx2(1,3)+rad2(3)^2*Rtrx2(1,1));
m23=(mass1f(2,3)+mass2f(2,3))+(rad1(3)*rad1(1)*Rtrx1(1,2)-rad1(3)*rad1(2)*Rtrx1(1,1)-...
    rad1(1)^2*Rtrx1(3,2)+rad1(1)*rad1(2)*Rtrx1(3,1))+...
    (rad2(3)*rad2(1)*Rtrx2(1,2)-rad2(3)*rad2(2)*Rtrx2(1,1)-rad2(1)^2*Rtrx2(3,2)+rad2(1)*rad2(2)*Rtrx2(3,1));
m33=(mass1f(3,3)+rad1(1)^2*Rtrx1(2,2)-2*rad1(1)*rad1(2)*Rtrx1(1,2)+rad1(2)^2*Rtrx1(1,1))+...
    (mass2f(3,3)+rad2(1)^2*Rtrx2(2,2)-2*rad2(1)*rad2(2)*Rtrx2(1,2)+rad2(2)^2*Rtrx2(1,1));
m21=m12;
m31=m13;
m32=m23;
m=[m11,m12,m13;m12,m22,m23;m13,m23,m33];
syms dv1 dv2 dv3 dp1 dp2 dp3
rm=inv(m);
% calculate the extreme case for stick together get the critical value miu
dv1=-rvl(1);
dv2=-rvl(2);
dv3=-rvl(3)*(1-res);
dp1=subs(rm(1,1)*dv1+rm(1,2)*dv2+rm(1,3)*dv3,{dv1,dv2,dv3},{-rvl(1),-rvl(2),-rvl(3)*(1-res)});
dp2=subs(rm(2,1)*dv1+rm(2,2)*dv2+rm(2,3)*dv3,{dv1,dv2,dv3},{-rvl(1),-rvl(2),-rvl(3)*(1-res)});
dp3=subs(rm(3,1)*dv1+rm(3,2)*dv2+rm(3,3)*dv3,{dv1,dv2,dv3},{-rvl(1),-rvl(2),-rvl(3)*(1-res)});
miu=sign(dp1)*sqrt(dp1^2+dp2^2)/dp3; % normal friction factor
miu2=dp2/dp1;
% friction matrix
flag='Stick';
if miu==0
    sm1=Inf;
else
    sm1=m11+m12*miu2+m13*sqrt(1+miu2*miu2)/miu;
    if miu2==0
        sm2=Inf;
    else
        sm2=m21/miu2+m22+m23*sqrt(1+miu2*miu2)/miu/miu2;
    end
end
sm3=m31*miu/sqrt(1+miu2*miu2)+m32*miu*miu2/sqrt(1+miu2*miu2)+m33;
if abs(miu)>=miu0 % sliding case
    flag='Slide';
    dv3=-rvl(3)*(1-res);
    fai=atan(miu2);
    if dp2==0
        fai=0/180*pi;
    end
    if miu0==0
        sm1=Inf;
    else
        sm1=m11+m12*miu2+m13*sqrt(1+miu2*miu2)/miu0;
        if miu2==0
            sm2=Inf;
        else
            sm2=m21/miu2+m22+m23*sqrt(1+miu2*miu2)/miu0/miu2;
        end
    end
    sm3=m31*miu0/sqrt(1+miu2*miu2)+m32*miu0*miu2/sqrt(1+miu2*miu2)+m33;
    AA=[miu0*cos(fai)*1e06 -rm(1,1) -rm(1,2);
        miu0*sin(fai)*1e06 -rm(2,1) -rm(2,2);
        1e06 -rm(3,1) -rm(3,2)];
    BB=[rm(1,3)*dv3 rm(2,3)*dv3 rm(3,3)*dv3]';
    CC=AA\BB;
    dp3=CC(1,1)*1e06;
    dv1=CC(2,1);
    dv2=CC(3,1);
    dp1=miu0*cos(fai)*dp3;
    dp2=miu0*sin(fai)*dp3;
end
%dpp=sqrt(dp1^2+dp2^2+dp3^2);
% energy on direction 1
%E1=abs(dp1/2.*(dv1+2*rvl(1)));
E1=abs(1/sm1/2*dv1*(dv1+2*rvl(1)));
% energy on direction 2
if miu2==0
    E2=0;
else
    %E2=abs(dp2/2.*(dv2+2*rvl(2)));
    E2=abs(1/sm2/2*dv2*(dv2+2*rvl(2)));    
end
%E3=abs(dp3/2.*(dv3+2*rvl(3)));
E3=abs(1/sm3/2*dv3*(dv3+2*rvl(3)));
% velocity change vector
dvv=[dv1;dv2;dv3]; % this is in local frame
% total energy
% do control to output if NaN, usually a result of Inf*0
if isnan(E1)==1
    E1=0;
elseif isnan(E2)==1
    E2=0;
elseif isnan(E3)==1
    E3=0;
end
tt=E1+E3+E2;
ttm=[E1, E2, E3];
%% compute the velocity after impact at body frame a and b
FF=mass1+mass2*inv(Mtr2)*Mlg;
SS=mass1*ve_a+mass2*ve_b;
QQ=mass2*inv(Mtr2)*(rvl+dvv);
ve_af=inv(FF)*(SS+QQ);                 % velocity of body a after impact under body frame a
ve_bf=inv(Mtr2)*(Mlg*ve_af-(rvl+dvv)); % velocity of body b after impact under body frame b
end